@@ -34,7 +34,6 @@ struct _MutterWindowPrivate
   ClutterActor     *shadow;
   Pixmap            back_pixmap;
 
-  MetaCompWindowType  type;
   Damage            damage;
 
   guint8            opacity;
@@ -42,10 +41,10 @@ struct _MutterWindowPrivate
   gchar *           desc;
 
   /* If the window is shaped, a region that matches the shape */
-  GdkRegion        *shape_region;
+  MetaRegion        *shape_region;
   /* A rectangular region with the unshaped extends of the window
    * texture */
-  GdkRegion        *bounding_region;
+  MetaRegion        *bounding_region;
 
   gint              freeze_count;
 
@@ -324,8 +323,6 @@ mutter_window_constructed (GObject *object)
 
   compositor = meta_display_get_compositor (display);
 
-  mutter_window_update_window_type (self);
-
 #ifdef HAVE_SHAPE
   /* Listen for ShapeNotify events on the window */
   if (meta_display_has_shape (display))
@@ -358,10 +355,6 @@ mutter_window_constructed (GObject *object)
     {
       priv->actor = mutter_shaped_texture_new ();
 
-      if (!clutter_glx_texture_pixmap_using_extension (
-                                  CLUTTER_GLX_TEXTURE_PIXMAP (priv->actor)))
-        g_warning ("NOTE: Not using GLX TFP!\n");
-
       clutter_container_add_actor (CLUTTER_CONTAINER (self), priv->actor);
 
       /*
@@ -539,13 +532,6 @@ mutter_window_get_property (GObject      *object,
     }
 }
 
-void
-mutter_window_update_window_type (MutterWindow *self)
-{
-  MutterWindowPrivate *priv = self->priv;
-  priv->type = (MetaCompWindowType) meta_window_get_window_type (priv->window);
-}
-
 static gboolean
 is_shaped (MetaDisplay *display, Window xwindow)
 {
@@ -569,6 +555,7 @@ static gboolean
 mutter_window_has_shadow (MutterWindow *self)
 {
   MutterWindowPrivate * priv = self->priv;
+  MetaWindowType window_type = meta_window_get_window_type (priv->window);
 
   if (priv->no_shadow)
     return FALSE;
@@ -622,17 +609,17 @@ mutter_window_has_shadow (MutterWindow *self)
   /*
    * Don't put shadow around DND icon windows
    */
-  if (priv->type == META_COMP_WINDOW_DND ||
-      priv->type == META_COMP_WINDOW_DESKTOP)
+  if (window_type == META_WINDOW_DND ||
+      window_type == META_WINDOW_DESKTOP)
     {
       meta_verbose ("Window 0x%x has no shadow as it is DND or Desktop\n",
 		    (guint)priv->xwindow);
       return FALSE;
     }
 
-  if (priv->type == META_COMP_WINDOW_MENU
+  if (window_type == META_WINDOW_MENU
 #if 0
-      || priv->type == META_COMP_WINDOW_DROPDOWN_MENU
+      || window_type == META_WINDOW_DROPDOWN_MENU
 #endif
       )
     {
@@ -642,7 +629,7 @@ mutter_window_has_shadow (MutterWindow *self)
     }
 
 #if 0
-  if (priv->type == META_COMP_WINDOW_TOOLTIP)
+  if (window_type == META_WINDOW_TOOLTIP)
     {
       meta_verbose ("Window 0x%x has shadow as it is a tooltip\n",
 		    (guint)priv->xwindow);
@@ -655,6 +642,10 @@ mutter_window_has_shadow (MutterWindow *self)
   return FALSE;
 }
 
+/**
+ * mutter_window_get_x_window: (skip)
+ *
+ */
 Window
 mutter_window_get_x_window (MutterWindow *self)
 {
@@ -690,15 +681,6 @@ mutter_window_get_texture (MutterWindow *self)
   return self->priv->actor;
 }
 
-MetaCompWindowType
-mutter_window_get_window_type (MutterWindow *self)
-{
-  if (!self)
-    return 0;
-
-  return self->priv->type;
-}
-
 gboolean
 mutter_window_is_override_redirect (MutterWindow *self)
 {
@@ -726,7 +708,7 @@ const char *mutter_window_get_description (MutterWindow *self)
 
 /**
  * mutter_window_get_workspace:
- * @self: #MutterWindow
+ * @mcw: #MutterWindow
  *
  * Returns the index of workspace on which this window is located; if the
  * window is sticky, or is not currently located on any workspace, returns -1.
@@ -1032,10 +1014,17 @@ mutter_window_detach (MutterWindow *self)
   if (!priv->back_pixmap)
     return;
 
-  XFreePixmap (xdisplay, priv->back_pixmap);
-  priv->back_pixmap = None;
+  /* Get rid of all references to the pixmap before freeing it; it's unclear whether
+   * you are supposed to be able to free a GLXPixmap after freeing the underlying
+   * pixmap, but it certainly doesn't work with current DRI/Mesa
+   */
   clutter_x11_texture_pixmap_set_pixmap (CLUTTER_X11_TEXTURE_PIXMAP (priv->actor),
                                          None);
+  mutter_shaped_texture_clear (MUTTER_SHAPED_TEXTURE (priv->actor));
+  cogl_flush();
+
+  XFreePixmap (xdisplay, priv->back_pixmap);
+  priv->back_pixmap = None;
 
   mutter_window_queue_create_pixmap (self);
 }
@@ -1046,10 +1035,12 @@ mutter_window_destroy (MutterWindow *self)
   MetaWindow	      *window;
   MetaCompScreen      *info;
   MutterWindowPrivate *priv;
+  MetaWindowType window_type;
 
   priv = self->priv;
 
   window = priv->window;
+  window_type = meta_window_get_window_type (window);
   meta_window_set_compositor_private (window, NULL);
 
   /*
@@ -1059,13 +1050,13 @@ mutter_window_destroy (MutterWindow *self)
   info = meta_screen_get_compositor_data (priv->screen);
   info->windows = g_list_remove (info->windows, (gconstpointer) self);
 
-  if (priv->type == META_COMP_WINDOW_DROPDOWN_MENU ||
-      priv->type == META_COMP_WINDOW_POPUP_MENU ||
-      priv->type == META_COMP_WINDOW_TOOLTIP ||
-      priv->type == META_COMP_WINDOW_NOTIFICATION ||
-      priv->type == META_COMP_WINDOW_COMBO ||
-      priv->type == META_COMP_WINDOW_DND ||
-      priv->type == META_COMP_WINDOW_OVERRIDE_OTHER)
+  if (window_type == META_WINDOW_DROPDOWN_MENU ||
+      window_type == META_WINDOW_POPUP_MENU ||
+      window_type == META_WINDOW_TOOLTIP ||
+      window_type == META_WINDOW_NOTIFICATION ||
+      window_type == META_WINDOW_COMBO ||
+      window_type == META_WINDOW_DND ||
+      window_type == META_WINDOW_OVERRIDE_OTHER)
     {
       /*
        * No effects, just kill it.
@@ -1350,7 +1341,7 @@ mutter_window_clear_shape_region (MutterWindow *self)
 
   if (priv->shape_region)
     {
-      gdk_region_destroy (priv->shape_region);
+      meta_region_destroy (priv->shape_region);
       priv->shape_region = NULL;
     }
 }
@@ -1362,7 +1353,7 @@ mutter_window_clear_bounding_region (MutterWindow *self)
 
   if (priv->bounding_region)
     {
-      gdk_region_destroy (priv->bounding_region);
+      meta_region_destroy (priv->bounding_region);
       priv->bounding_region = NULL;
     }
 }
@@ -1377,7 +1368,7 @@ mutter_window_update_bounding_region (MutterWindow *self,
 
   mutter_window_clear_bounding_region (self);
 
-  priv->bounding_region = gdk_region_rectangle (&bounding_rectangle);
+  priv->bounding_region = meta_region_new_from_rectangle (&bounding_rectangle);
 }
 
 static void
@@ -1390,11 +1381,11 @@ mutter_window_update_shape_region (MutterWindow *self,
 
   mutter_window_clear_shape_region (self);
 
-  priv->shape_region = gdk_region_new ();
+  priv->shape_region = meta_region_new ();
   for (i = 0; i < n_rects; i++)
     {
       GdkRectangle rect = { rects[i].x, rects[i].y, rects[i].width, rects[i].height };
-      gdk_region_union_with_rect (priv->shape_region, &rect);
+      meta_region_union_rectangle (priv->shape_region, &rect);
     }
 }
 
@@ -1408,7 +1399,7 @@ mutter_window_update_shape_region (MutterWindow *self,
  * Return value: (transfer none): the area obscured by the window,
  *  %NULL is the same as an empty region.
  */
-GdkRegion *
+MetaRegion *
 mutter_window_get_obscured_region (MutterWindow *self)
 {
   MutterWindowPrivate *priv = self->priv;
@@ -1427,13 +1418,13 @@ mutter_window_get_obscured_region (MutterWindow *self)
 #if 0
 /* Print out a region; useful for debugging */
 static void
-dump_region (GdkRegion *region)
+dump_region (MetaRegion *region)
 {
   GdkRectangle *rects;
   int n_rects;
   int i;
 
-  gdk_region_get_rectangles (region, &rects, &n_rects);
+  meta_region_get_rectangles (region, &rects, &n_rects);
   g_print ("[");
   for (i = 0; i < n_rects; i++)
     {
@@ -1457,10 +1448,10 @@ dump_region (GdkRegion *region)
  */
 void
 mutter_window_set_visible_region (MutterWindow *self,
-                                  GdkRegion    *visible_region)
+                                  MetaRegion   *visible_region)
 {
   MutterWindowPrivate *priv = self->priv;
-  GdkRegion *texture_clip_region = NULL;
+  MetaRegion *texture_clip_region = NULL;
 
   /* Get the area of the window texture that would be drawn if
    * we weren't obscured at all
@@ -1468,21 +1459,21 @@ mutter_window_set_visible_region (MutterWindow *self,
   if (priv->shaped)
     {
       if (priv->shape_region)
-        texture_clip_region = gdk_region_copy (priv->shape_region);
+        texture_clip_region = meta_region_copy (priv->shape_region);
     }
   else
     {
       if (priv->bounding_region)
-        texture_clip_region = gdk_region_copy (priv->bounding_region);
+        texture_clip_region = meta_region_copy (priv->bounding_region);
     }
 
   if (!texture_clip_region)
-    texture_clip_region = gdk_region_new ();
+    texture_clip_region = meta_region_new ();
 
   /* Then intersect that with the visible region to get the region
    * that we actually need to redraw.
    */
-  gdk_region_intersect (texture_clip_region, visible_region);
+  meta_region_intersect (texture_clip_region, visible_region);
 
   /* Assumes ownership */
   mutter_shaped_texture_set_clip_region (MUTTER_SHAPED_TEXTURE (priv->actor),
@@ -1503,7 +1494,7 @@ mutter_window_set_visible_region (MutterWindow *self,
  */
 void
 mutter_window_set_visible_region_beneath (MutterWindow *self,
-                                          GdkRegion    *beneath_region)
+                                          MetaRegion    *beneath_region)
 {
   MutterWindowPrivate *priv = self->priv;
 
@@ -1511,7 +1502,7 @@ mutter_window_set_visible_region_beneath (MutterWindow *self,
     {
       GdkRectangle shadow_rect;
       ClutterActorBox box;
-      GdkOverlapType overlap;
+      MetaOverlapType overlap;
 
       /* We could compute an full clip region as we do for the window
        * texture, but the shadow is relatively cheap to draw, and
@@ -1526,10 +1517,10 @@ mutter_window_set_visible_region_beneath (MutterWindow *self,
       shadow_rect.width = roundf (box.x2 - box.x1);
       shadow_rect.height = roundf (box.y2 - box.y1);
 
-      overlap = gdk_region_rect_in (beneath_region, &shadow_rect);
+      overlap = meta_region_contains_rectangle (beneath_region, &shadow_rect);
 
       tidy_texture_frame_set_needs_paint (TIDY_TEXTURE_FRAME (priv->shadow),
-                                          overlap != GDK_OVERLAP_RECTANGLE_OUT);
+                                          overlap != META_REGION_OVERLAP_OUT);
     }
 }
 
@@ -1610,17 +1601,21 @@ check_needs_pixmap (MutterWindow *self)
           return;
         }
 
-      /* MUST call before setting pixmap or serious performance issues
-       * seemingly caused by cogl_texture_set_filters() in set_filter
-       * Not sure if that call is actually needed.
-       */
-      if (!compositor->no_mipmaps)
-        clutter_texture_set_filter_quality (CLUTTER_TEXTURE (priv->actor),
-                                            CLUTTER_TEXTURE_QUALITY_HIGH );
+      if (compositor->no_mipmaps)
+        mutter_shaped_texture_set_create_mipmaps (MUTTER_SHAPED_TEXTURE (priv->actor),
+                                                  FALSE);
 
       clutter_x11_texture_pixmap_set_pixmap
                        (CLUTTER_X11_TEXTURE_PIXMAP (priv->actor),
                         priv->back_pixmap);
+      /*
+       * This only works *after* actually setting the pixmap, so we have to
+       * do it here.
+       * See: http://bugzilla.clutter-project.org/show_bug.cgi?id=2236
+       */
+      if (!clutter_glx_texture_pixmap_using_extension (
+                                  CLUTTER_GLX_TEXTURE_PIXMAP (priv->actor)))
+        g_warning ("NOTE: Not using GLX TFP!\n");
 
       g_object_get (priv->actor,
                     "pixmap-width", &pxm_width,
@@ -1701,6 +1696,8 @@ static void
 check_needs_reshape (MutterWindow *self)
 {
   MutterWindowPrivate *priv = self->priv;
+  MetaScreen *screen = priv->screen;
+  MetaDisplay *display = meta_screen_get_display (screen);
 
   if (!priv->needs_reshape)
     return;
@@ -1711,15 +1708,17 @@ check_needs_reshape (MutterWindow *self)
 #ifdef HAVE_SHAPE
   if (priv->shaped)
     {
-      Display *xdisplay = meta_display_get_xdisplay (meta_window_get_display (priv->window));
+      Display *xdisplay = meta_display_get_xdisplay (display);
       XRectangle *rects;
       int n_rects, ordering;
 
+      meta_error_trap_push (display);
       rects = XShapeGetRectangles (xdisplay,
                                    priv->xwindow,
                                    ShapeBounding,
                                    &n_rects,
                                    &ordering);
+      meta_error_trap_pop (display, TRUE);
 
       if (rects)
         {
@@ -1765,7 +1764,9 @@ mutter_window_pre_paint (MutterWindow *self)
 
   if (priv->received_damage)
     {
+      meta_error_trap_push (display);
       XDamageSubtract (xdisplay, priv->damage, None, None);
+      meta_error_trap_pop (display, FALSE);
       priv->received_damage = FALSE;
     }
 
