@@ -401,7 +401,7 @@ static GSourceFuncs event_funcs = {
 static void
 meta_clutter_init (GOptionContext *ctx, int *argc, char ***argv)
 {
-  clutter_x11_set_display (gdk_display);
+  clutter_x11_set_display (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   clutter_x11_disable_event_retrieval ();
   
   if (CLUTTER_INIT_SUCCESS == clutter_init (argc, argv))
@@ -586,7 +586,7 @@ main (int argc, char **argv)
            * is initialized at this point, and we don't plan to run any real
            * plugin code.
            */
-          MutterPluginManager *mgr = mutter_plugin_manager_new (NULL);
+          MutterPluginManager *mgr = mutter_plugin_manager_get_default ();
           if (!mutter_plugin_manager_load (mgr))
             g_critical ("failed to load plugins");
         }
