@@ -1039,6 +1039,7 @@ meta_preference_get_type (void)
         { META_PREF_MOUSE_BUTTON_MODS, "META_PREF_MOUSE_BUTTON_MODS", "mouse-button-mods" },
         { META_PREF_FOCUS_MODE, "META_PREF_FOCUS_MODE", "focus-mode" },
         { META_PREF_FOCUS_NEW_WINDOWS, "META_PREF_FOCUS_NEW_WINDOWS", "focus-new-windows" },
+        { META_PREF_ATTACH_MODAL_DIALOGS, "META_PREF_ATTACH_MODAL_DIALOGS", "attach-modal-dialogs" },
         { META_PREF_RAISE_ON_CLICK, "META_PREF_RAISE_ON_CLICK", "raise-on-click" },
         { META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR, "META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR", "action-double-click-titlebar" },
         { META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR, "META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR", "action-middle-click-titlebar" },
@@ -1064,6 +1065,7 @@ meta_preference_get_type (void)
         { META_PREF_CURSOR_SIZE, "META_PREF_CURSOR_SIZE", "cursor-size" },
         { META_PREF_COMPOSITING_MANAGER, "META_PREF_COMPOSITING_MANAGER", "compositing-manager" },
         { META_PREF_RESIZE_WITH_RIGHT_BUTTON, "META_PREF_RESIZE_WITH_RIGHT_BUTTON", "resize-with-right-button" },
+        { META_PREF_SIDE_BY_SIDE_TILING, "META_PREF_SIDE_BY_SIDE_TILING", "side-by-side-tiling" },
         { META_PREF_FORCE_FULLSCREEN, "META_PREF_FORCE_FULLSCREEN", "force-fullscreen" },
         { META_PREF_CLUTTER_PLUGINS, "META_PREF_CLUTTER_PLUGINS", "clutter-plugins" },
         { META_PREF_LIVE_HIDDEN_WINDOWS, "META_PREF_LIVE_HIDDEN_WINDOWS", "live-hidden-windows" },
@@ -1233,42 +1235,6 @@ meta_maximize_flags_get_type (void)
 /* enumerations from "include/compositor.h" */
 #include "include/compositor.h"
 
-GType
-meta_comp_window_type_get_type (void)
-{
-  static volatile gsize g_enum_type_id__volatile = 0;
-
-  if (g_once_init_enter (&g_enum_type_id__volatile))
-    {
-      static const GEnumValue values[] = {
-        { META_COMP_WINDOW_NORMAL, "META_COMP_WINDOW_NORMAL", "normal" },
-        { META_COMP_WINDOW_DESKTOP, "META_COMP_WINDOW_DESKTOP", "desktop" },
-        { META_COMP_WINDOW_DOCK, "META_COMP_WINDOW_DOCK", "dock" },
-        { META_COMP_WINDOW_DIALOG, "META_COMP_WINDOW_DIALOG", "dialog" },
-        { META_COMP_WINDOW_MODAL_DIALOG, "META_COMP_WINDOW_MODAL_DIALOG", "modal-dialog" },
-        { META_COMP_WINDOW_TOOLBAR, "META_COMP_WINDOW_TOOLBAR", "toolbar" },
-        { META_COMP_WINDOW_MENU, "META_COMP_WINDOW_MENU", "menu" },
-        { META_COMP_WINDOW_UTILITY, "META_COMP_WINDOW_UTILITY", "utility" },
-        { META_COMP_WINDOW_SPLASHSCREEN, "META_COMP_WINDOW_SPLASHSCREEN", "splashscreen" },
-        { META_COMP_WINDOW_DROPDOWN_MENU, "META_COMP_WINDOW_DROPDOWN_MENU", "dropdown-menu" },
-        { META_COMP_WINDOW_POPUP_MENU, "META_COMP_WINDOW_POPUP_MENU", "popup-menu" },
-        { META_COMP_WINDOW_TOOLTIP, "META_COMP_WINDOW_TOOLTIP", "tooltip" },
-        { META_COMP_WINDOW_NOTIFICATION, "META_COMP_WINDOW_NOTIFICATION", "notification" },
-        { META_COMP_WINDOW_COMBO, "META_COMP_WINDOW_COMBO", "combo" },
-        { META_COMP_WINDOW_DND, "META_COMP_WINDOW_DND", "dnd" },
-        { META_COMP_WINDOW_OVERRIDE_OTHER, "META_COMP_WINDOW_OVERRIDE_OTHER", "override-other" },
-        { 0, NULL, NULL }
-      };
-      GType g_enum_type_id;
-
-      g_enum_type_id =
-        g_enum_register_static (g_intern_static_string ("MetaCompWindowType"), values);
-
-      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
-    }
-
-  return g_enum_type_id__volatile;
-}
 GType
 meta_comp_effect_get_type (void)
 {
