@@ -25,6 +25,8 @@
 #include "fixedtip.h"
 #include "ui.h"
 
+#include "gdk2-drawing-utils.h"
+
 /**
  * The floating rectangle.  This is a GtkWindow, and it contains
  * the "label" widget, below.
@@ -50,8 +52,24 @@ static int screen_right_edge = 0;
  */
 static int screen_bottom_edge = 0;
 
+#ifdef USE_GTK3
+static gboolean
+draw_handler (GtkWidget *tooltips,
+              cairo_t   *cr)
+{
+  gtk_paint_flat_box (gtk_widget_get_style (tip),
+                      cr,
+                      GTK_STATE_NORMAL, GTK_SHADOW_OUT, 
+                      tip, "tooltip",
+                      0, 0,
+                      gtk_widget_get_allocated_width (tooltips),
+                      gtk_widget_get_allocated_height (tooltips));
+
+  return FALSE;
+}
+#else /* !USE_GTK3 */
 static gint
-expose_handler (GtkTooltips *tooltips)
+expose_handler (GtkWidget *tooltips)
 {
   gtk_paint_flat_box (gtk_widget_get_style (tip),
                       gtk_widget_get_window (tip),
@@ -61,6 +79,7 @@ expose_handler (GtkTooltips *tooltips)
 
   return FALSE;
 }
+#endif /* !USE_GTK3 */
 
 void
 meta_fixed_tip_show (Display *xdisplay, int screen_number,
@@ -94,8 +113,13 @@ meta_fixed_tip_show (Display *xdisplay, int screen_number,
       gtk_widget_set_name (tip, "gtk-tooltips");
       gtk_container_set_border_width (GTK_CONTAINER (tip), 4);
 
+#ifdef USE_GTK3
+      g_signal_connect_swapped (tip, "draw",
+				 G_CALLBACK (draw_handler), NULL);
+#else
       g_signal_connect_swapped (tip, "expose_event",
 				 G_CALLBACK (expose_handler), NULL);
+#endif
 
       label = gtk_label_new (NULL);
       gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
