@@ -32,11 +32,13 @@
  */
 #include "../core/workspace-private.h"
 #include "../core/frame-private.h"
+#include "region.h"
 #include "draw-workspace.h"
 #include <gtk/gtk.h>
 #include <math.h>
 
 #include "gtk-compat.h"
+#include "gdk2-drawing-utils.h"
 
 #define OUTSIDE_SELECT_RECT 2
 #define INSIDE_SELECT_RECT 2
@@ -73,15 +75,24 @@ static GtkWidget* selectable_workspace_new (MetaWorkspace *workspace);
 static void       select_workspace         (GtkWidget *widget);
 static void       unselect_workspace       (GtkWidget *widget);
 
+#ifdef USE_GTK3
+static gboolean
+outline_window_draw (GtkWidget *widget,
+                     cairo_t   *cr,
+                     gpointer   data)
+{
+#else /* !USE_GTK3 */
 static gboolean
 outline_window_expose (GtkWidget      *widget,
                        GdkEventExpose *event,
                        gpointer        data)
 {
+  cairo_t *cr = gdk_cairo_create (event->window);
+#endif /* !USE_GTK3 */
+
   MetaTabPopup *popup;
   TabEntry *te;
   GtkStyle *style;
-  GdkWindow *window;
   
   popup = data;
 
@@ -89,22 +100,26 @@ outline_window_expose (GtkWidget      *widget,
     return FALSE;
 
   te = popup->current_selected_entry;
-  window = gtk_widget_get_window (widget);
   style = gtk_widget_get_style (widget);
-  
-  gdk_draw_rectangle (window,
-                      style->white_gc,
-                      FALSE,
-                      0, 0,
-                      te->rect.width - 1,
-                      te->rect.height - 1);
 
-  gdk_draw_rectangle (window,
-                      style->white_gc,
-                      FALSE,
-                      te->inner_rect.x - 1, te->inner_rect.y - 1,
-                      te->inner_rect.width + 1,
-                      te->inner_rect.height + 1);
+  cairo_set_line_width (cr, 1.0);
+  gdk_cairo_set_source_color (cr, &style->white);
+
+  cairo_rectangle (cr,
+                   0.5, 0.5,
+                   te->rect.width - 1,
+                   te->rect.height - 1);
+  cairo_stroke (cr);
+
+  cairo_rectangle (cr,
+                   te->inner_rect.x - 0.5, te->inner_rect.y - 0.5,
+                   te->inner_rect.width + 1,
+                   te->inner_rect.height + 1);
+  cairo_stroke (cr);
+
+#ifndef USE_GTK3
+  cairo_destroy (cr);
+#endif
 
   return FALSE;
 }
@@ -246,8 +261,13 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   gtk_widget_set_app_paintable (popup->outline_window, TRUE);
   gtk_widget_realize (popup->outline_window);
 
+#ifdef USE_GTK3
+  g_signal_connect (G_OBJECT (popup->outline_window), "draw",
+                    G_CALLBACK (outline_window_draw), popup);
+#else
   g_signal_connect (G_OBJECT (popup->outline_window), "expose_event",
                     G_CALLBACK (outline_window_expose), popup);
+#endif
   
   popup->window = gtk_window_new (GTK_WINDOW_POPUP);
 
@@ -449,7 +469,8 @@ meta_ui_tab_popup_set_showing (MetaTabPopup *popup,
         {
           meta_verbose ("Hiding tab popup window\n");
           gtk_widget_hide (popup->window);
-          meta_core_increment_event_serial (gdk_display);
+          meta_core_increment_event_serial (
+              GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
         }
     }
 }
@@ -459,9 +480,9 @@ display_entry (MetaTabPopup *popup,
                TabEntry     *te)
 {
   GdkRectangle rect;
-  GdkRegion *region;
-  GdkRegion *inner_region;
   GdkWindow *window;
+  MetaRegion *region;
+  MetaRegion *inner_region;
 
   
   if (popup->current_selected_entry)
@@ -485,7 +506,8 @@ display_entry (MetaTabPopup *popup,
 
       /* Do stuff behind gtk's back */
       gdk_window_hide (window);
-      meta_core_increment_event_serial (gdk_display);
+      meta_core_increment_event_serial (
+          GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   
       rect = te->rect;
       rect.x = 0;
@@ -498,16 +520,16 @@ display_entry (MetaTabPopup *popup,
       gdk_window_set_background (window,
                                  &gtk_widget_get_style (popup->outline_window)->black);
   
-      region = gdk_region_rectangle (&rect);
-      inner_region = gdk_region_rectangle (&te->inner_rect);
-      gdk_region_subtract (region, inner_region);
-      gdk_region_destroy (inner_region);
+      region = meta_region_new_from_rectangle (&rect);
+      inner_region = meta_region_new_from_rectangle (&te->inner_rect);
+      meta_region_subtract (region, inner_region);
+      meta_region_destroy (inner_region);
   
       gdk_window_shape_combine_region (window,
                                        region,
                                        0, 0);
 
-      gdk_region_destroy (region);
+      meta_region_destroy (region);
   
       /* This should piss off gtk a bit, but we don't want to raise
        * above the tab popup.  So, instead of calling gtk_widget_show,
@@ -651,8 +673,13 @@ unselect_image (GtkWidget *widget)
 }
 
 static void     meta_select_image_class_init   (MetaSelectImageClass *klass);
+#if USE_GTK3
+static gboolean meta_select_image_draw         (GtkWidget            *widget,
+                                                cairo_t              *cr);
+#else
 static gboolean meta_select_image_expose_event (GtkWidget            *widget,
                                                 GdkEventExpose       *event);
+#endif
 
 static GtkImageClass *parent_class;
 
@@ -691,16 +718,37 @@ meta_select_image_class_init (MetaSelectImageClass *klass)
 
   widget_class = GTK_WIDGET_CLASS (klass);
   
+#if USE_GTK3
+  widget_class->draw = meta_select_image_draw;
+#else
   widget_class->expose_event = meta_select_image_expose_event;
+#endif
 }
 
+#if USE_GTK3
+static gboolean
+meta_select_image_draw (GtkWidget *widget,
+                        cairo_t   *cr)
+{
+  GtkAllocation allocation;
+
+  gtk_widget_get_allocation (widget, &allocation);
+#else /* !USE_GTK3 */
 static gboolean
 meta_select_image_expose_event (GtkWidget      *widget,
                                 GdkEventExpose *event)
 {
+  GtkAllocation allocation;
+  cairo_t *cr = gdk_cairo_create (event->window);
+
+  gdk_cairo_region (cr, event->region);
+  cairo_clip (cr);
+  gtk_widget_get_allocation (widget, &allocation);
+  cairo_translate (cr, allocation.x, allocation.y);
+#endif
+
   if (META_SELECT_IMAGE (widget)->selected)
     {
-      GtkAllocation allocation;
       GtkMisc *misc;
       GtkRequisition requisition;
       GtkStyle *style;
@@ -712,17 +760,12 @@ meta_select_image_expose_event (GtkWidget      *widget,
 
       misc = GTK_MISC (widget);
 
-      gtk_widget_get_allocation (widget, &allocation);
       gtk_widget_get_requisition (widget, &requisition);
       gtk_misc_get_alignment (misc, &xalign, &yalign);
       gtk_misc_get_padding (misc, &xpad, &ypad);
       
-      x = (allocation.x * (1.0 - xalign) +
-           (allocation.x + allocation.width
-            - (requisition.width - xpad * 2)) * xalign) + 0.5;
-      y = (allocation.y * (1.0 - yalign) +
-           (allocation.y + allocation.height
-            - (requisition.height - ypad * 2)) * yalign) + 0.5;
+      x = (allocation.width - (requisition.width - xpad * 2)) * xalign + 0.5;
+      y = (allocation.height - (requisition.height - ypad * 2)) * yalign + 0.5;
 
       x -= INSIDE_SELECT_RECT + 1;
       y -= INSIDE_SELECT_RECT + 1;      
@@ -734,29 +777,22 @@ meta_select_image_expose_event (GtkWidget      *widget,
       style = gtk_widget_get_style (widget);
       state = gtk_widget_get_state (widget);
 
-      gdk_draw_rectangle (window,
-                          style->fg_gc[state],
-                          FALSE,
-                          x, y, w, h);
-      gdk_draw_rectangle (window,
-                          style->fg_gc[state],
-                          FALSE,
-                          x - 1, y - 1, w + 2, h + 2);
-      
-#if 0
-      gdk_draw_rectangle (widget->window,
-                          widget->style->bg_gc[GTK_STATE_SELECTED],
-                          TRUE,
-                          x, y, w, h);
-#endif
-#if 0      
-      gtk_paint_focus (widget->style, widget->window,
-                       &event->area, widget, "meta-tab-image",
-                       x, y, w, h);
-#endif
+      cairo_set_line_width (cr, 2.0);
+      gdk_cairo_set_source_color (cr, &style->fg[state]);
+
+      cairo_rectangle (cr, x, y, w + 1, h + 1);
+      cairo_stroke (cr);
+
+      cairo_set_line_width (cr, 1.0);
     }
 
+#ifdef USE_GTK3
+  return GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
+#else
+  cairo_destroy (cr);
+
   return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+#endif /* !USE_GTK3 */
 }
 
 #define META_TYPE_SELECT_WORKSPACE   (meta_select_workspace_get_type ())
@@ -820,8 +856,13 @@ unselect_workspace (GtkWidget *widget)
 
 static void meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass);
 
+#if USE_GTK3
+static gboolean meta_select_workspace_draw         (GtkWidget      *widget,
+                                                    cairo_t        *cr);
+#else
 static gboolean meta_select_workspace_expose_event (GtkWidget      *widget,
                                                     GdkEventExpose *event);
+#endif
 
 GType
 meta_select_workspace_get_type (void)
@@ -859,7 +900,11 @@ meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass)
   
   widget_class = GTK_WIDGET_CLASS (klass);
   
+#if USE_GTK3
+  widget_class->draw = meta_select_workspace_draw;
+#else
   widget_class->expose_event = meta_select_workspace_expose_event;
+#endif
 }
 
 /**
@@ -896,15 +941,22 @@ meta_convert_meta_to_wnck (MetaWindow *window, MetaScreen *screen)
 }
 
 
+#ifdef USE_GTK3
+static gboolean
+meta_select_workspace_draw (GtkWidget *widget,
+                            cairo_t   *cr)
+{
+#else /* !USE_GTK3 */
 static gboolean
 meta_select_workspace_expose_event (GtkWidget      *widget,
                                     GdkEventExpose *event)
 {
+  cairo_t *cr = gdk_cairo_create (event->window);
+#endif /* !USE_GTK3 */
   MetaWorkspace *workspace;
   WnckWindowDisplayInfo *windows;
   GtkAllocation allocation;
   GtkStyle *style;
-  GdkWindow *window;
   int i, n_windows;
   GList *tmp, *list;
 
@@ -943,11 +995,10 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
 
   g_list_free (list);
 
-  window = gtk_widget_get_window (widget);
   gtk_widget_get_allocation (widget, &allocation);
 
   wnck_draw_workspace (widget,
-                       window,
+                       cr,
                        SELECT_OUTLINE_WIDTH,
                        SELECT_OUTLINE_WIDTH,
                        allocation.width - SELECT_OUTLINE_WIDTH * 2,
@@ -964,22 +1015,21 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
   if (META_SELECT_WORKSPACE (widget)->selected)
     {
       style = gtk_widget_get_style (widget);
-      i = SELECT_OUTLINE_WIDTH - 1;
 
-      while (i >= 0)
-        {
-          gdk_draw_rectangle (window,
-                              style->fg_gc[gtk_widget_get_state (widget)],
-                              FALSE,
-                              i, 
-                              i,
-                              allocation.width - i * 2 - 1,
-                              allocation.height - i * 2 - 1);
+      gdk_cairo_set_source_color (cr,
+                                  &style->fg[gtk_widget_get_state (widget)]);
+      cairo_set_line_width (cr, SELECT_OUTLINE_WIDTH);
 
-          --i;
-        }
+      cairo_rectangle (cr,
+                       SELECT_OUTLINE_WIDTH / 2.0, SELECT_OUTLINE_WIDTH / 2.0,
+                       allocation.width - SELECT_OUTLINE_WIDTH,
+                       allocation.height - SELECT_OUTLINE_WIDTH);
+      cairo_stroke (cr);
     }
 
+#ifndef USE_GTK3
+  cairo_destroy (cr);
+#endif
+
   return TRUE;
 }
-
